class IssueService:

    def __init__(self, app):
        """
        :type app: metasdk.MetaApp
        """
        self.__app = app
        self.__options = {}
        self.__data_get_cache = {}
        self.__metadb = app.db("meta")

    def change_issue_status(self, issue_id, status_id: str):
        """
        Смета статуса тикета
        :param issue_id: int
        :param status_id: int
        """
        self.__metadb.update("""
              update meta.issue set 
                issue_status_id=:status_id,
                assignee_user_id=valera_user_id(),
                last_user_id=valera_user_id()
              where id = :issue_id
        """, {"issue_id": issue_id, "status_id": status_id})

    def pending_issue(self, issue_id):
        """
        Перевести в статус "В ожидании"
        :param issue_id: int
        """
        self.change_issue_status(issue_id, '1')

    def in_progress_issue(self, issue_id):
        """
        Взять в работу
        :param issue_id: int
        """
        self.change_issue_status(issue_id, '2')

    def reject_issue(self, issue_id):
        """
        Отклонить задачу
        :param issue_id: int
        """
        self.change_issue_status(issue_id, '7')

    def clarification_issue(self, issue_id):
        """
        Уточнение
        :param issue_id: int
        """
        self.change_issue_status(issue_id, '4')

    def done_issue(self, issue_id):
        """
        Успешное выполенение
        :param issue_id: int
        """
        self.change_issue_status(issue_id, '3')

    def add_issue_msg(self, issue_id, msg):
        self.__metadb.update("""
            INSERT INTO meta.issue_msg (msg, issue_id, user_id, last_user_id)
            VALUES (:msg, :issue_id, valera_user_id(), valera_user_id())
        """, {"issue_id": issue_id, "msg": msg})



import json


class MediaService:
    def __init__(self, app):
        """
        :type app: metasdk.MetaApp
        """
        self.__app = app
        self.__options = {}

    def persist_one(self, file_base64_content, filename, extension, mime, is_private=True):
        """
        Загружает файл в облако
        :type origin: string Принимает значения ROBOT, USER
        """
        return self.__app.api_call("MediaService", "persist_one", locals(), {})

    def upload(self, file_descriptor, settings):
        """
        Загружает файл в облако
        :param file_descriptor: открытый дескриптор
        :param settings: настройки загрузки
        :rtype: requests.Response
        """
        multipart_form_data = {
            'file': file_descriptor
        }
        params = {"settings": json.dumps(settings)}
        dr = self.__app.native_api_call('media', 'upload', params, self.__options, True, multipart_form_data, False, http_path="/api/meta/v1/", http_method='POST',
                                        connect_timeout_sec=60 * 10)
        return json.loads(dr.text)

    def download(self, media_id, as_stream=False):
        """
        Скачивает указанный файл
        :param media_id: string
        :rtype: requests.Response
        """
        response = self.__app.native_api_call('media', 'd/' + media_id, {}, self.__options, False, None, as_stream, http_path="/api/meta/v1/", http_method='GET')
        return response

    def info(self, media_id):
        """
        Получить информацию по файлу
        :param media_id:
        :rtype: requests.Response
        """
        dr = self.__app.native_api_call('media', 'i/' + media_id, {}, self.__options, False, None, False, http_path="/api/meta/v1/", http_method='GET')
        return json.loads(dr.text)



from metasdk.exceptions import AuthError, ServerError, RequestError, UnexpectedError, NoContentError


def get_api_call_headers(app):
    """
    Генерирует заголовки для API запроса.
    Тут же подкладывается авторизация

    :type app: metasdk.MetaApp
    """
    headers = {
        "content-type": "application/json;charset=UTF-8",
        "User-Agent": app.user_agent,
    }
    if not app.developer_settings:
        raise AuthError({"message": "Для корректной работы SDK нужно установить настройки разработчика", "url": "https://apps.devision.io/page?a=63&p=3975"})
    headers.update(app.developer_settings.get('api_headers'))
    return headers


def process_meta_api_error_code(status_code, request, response_text):
    if status_code == 204:
        raise NoContentError(request)
    elif status_code == 401:
        raise AuthError(request)
    elif status_code >= 500:
        raise ServerError(response_text)
    elif status_code >= 400:
        raise RequestError(response_text)
    else:
        raise UnexpectedError(request)



from ..utils import decode_jwt


class ExternalSystemService:
    def __init__(self, app):
        """
        :type app: metasdk.MetaApp
        """
        self.__app = app
        self.__options = {}
        self.__data_get_cache = {}
        self.__data_get_flatten_cache = {}
        self.__metadb = app.db("meta")
        self.__crypt_params = None

    def discovery_access(self, company_id: int, ex_system_id: str, access_login: str):
        """
        Удобно, когда у вас есть понимаение кода надо сделать запрос, но нет ex_access_id.
        Например интеграторы
        :param company_id:
        :param ex_system_id:
        :param access_login:
        :return:
        """
        ex_access = self.__metadb.one(
            """
            SELECT
                ex_system_id, 
                login, 
                token_info,
                form_data 
            FROM meta.ex_access 
            WHERE login = :access_login
                AND ex_system_id = :ex_system_id
                AND company_id = :company_id
            """,
            {
                "access_login": access_login,
                "ex_system_id": ex_system_id,
                "company_id": company_id
            }
        )

        return self.__prep_ex_access(ex_access)

    def get_access(self, ex_access_id):
        ex_access = self.__metadb.one(
            """
            SELECT
                ex_system_id, 
                login, 
                token_info,
                form_data 
            FROM meta.ex_access 
            WHERE id=:id::uuid
            """,
            {"id": ex_access_id}
        )

        return self.__prep_ex_access(ex_access)

    def __prep_ex_access(self, ex_access):
        if ex_access is None:
            return None

        token_info_ = ex_access.get('token_info')

        if token_info_ and token_info_.get('accessToken'):
            ex_access['token_info']['accessToken'] = decode_jwt(token_info_.get('accessToken'), self.__get_crypt_params()['secureKey'])

        if token_info_ and token_info_.get('refreshToken'):
            ex_access['token_info']['refreshToken'] = decode_jwt(token_info_.get('refreshToken'), self.__get_crypt_params()['secureKey'])

        return ex_access

    def __get_crypt_params(self):
        # lazy load
        if self.__crypt_params is None:
            self.__crypt_params = self.__app.SettingsService.data_get("crypt_params")
        return self.__crypt_params
